use std::str::Utf8Error;

use quick_xml::events::attributes::AttrError;
use rsasl::mechname::MechanismNameError;

use crate::{
    jid::ParseError,
    stanza::{self, ElementError},
};

#[derive(Debug)]
pub enum JabberError {
    Connection,
    BadStream,
    StartTlsUnavailable,
    TlsNegotiation,
    Utf8Decode,
    NoFeatures,
    UnknownNamespace,
    ParseError,
    XML(quick_xml::Error),
    SASL(SASLError),
    Element(ElementError<'static>),
    JID(ParseError),
}

#[derive(Debug)]
pub enum SASLError {
    SASL(rsasl::prelude::SASLError),
    MechanismName(MechanismNameError),
}

impl From<rsasl::prelude::SASLError> for JabberError {
    fn from(e: rsasl::prelude::SASLError) -> Self {
        Self::SASL(SASLError::SASL(e))
    }
}

impl From<MechanismNameError> for JabberError {
    fn from(value: MechanismNameError) -> Self {
        Self::SASL(SASLError::MechanismName(value))
    }
}

impl From<Utf8Error> for JabberError {
    fn from(_e: Utf8Error) -> Self {
        Self::Utf8Decode
    }
}

impl From<quick_xml::Error> for JabberError {
    fn from(e: quick_xml::Error) -> Self {
        Self::XML(e)
    }
}

impl From<stanza::ElementError<'static>> for JabberError {
    fn from(e: stanza::ElementError<'static>) -> Self {
        Self::Element(e)
    }
}

impl From<AttrError> for JabberError {
    fn from(e: AttrError) -> Self {
        Self::XML(e.into())
    }
}

impl From<ParseError> for JabberError {
    fn from(e: ParseError) -> Self {
        Self::JID(e)
    }
}
